

--- Visualisation library for TerraME
--  This module implements functions to create a color bar using the Color Brewer
--  @author Gilberto Camara, 2014

--- Configuration variable for contributed TerraME libraries
--  @usage tmepath should point to the TerraME installation directory
tmepath = os.getenv("TME_PATH_1_3_0")
package.path = package.path .. ";"..tmepath.."/contrib/viewerlib/?.lua;"

--- Dependencies of the color bar
--  @usage brew calls the Color Brewer module
local brewer = require ("brewer")

local colorbar = {}


--- function to return an index for the color names
--- @param name a name of a color palette in Brewer

colorbar.index = function (colorname)
    if  (brewer.rgb[colorname]  ~= nil) then
        return colorname
    else 
        local names = {}
        for i = 1, #(brewer.matchnames) do
            names = brewer.matchnames [i]
            for j = 1, #(names) do
                if (colorname == names [j]) then
                    return names [1]
                end
            end
        end
    end
    error("Viewer: color names must exist in Color Brewer")
return nil
end
    
--- function to create a color bar using Color Brewer 
--  returns a valid colorbar for TerraME
--  @param values the set of values that will receive colors
--  @param colornames a valid set of color names for the Color Brewer
--  @param labels to be used for the colors (optional)
--  @usage view.uniqueValueColorBar (values, colornames, labels)
--
--  colorbar.brewer ({0,1,2,3,4}, "YellowishGreens")
--  colorbar.brewer ({0,1,2}, "YlGn", {"zero", "one", "two"})
--  
colorbar.brewer = function (values, name, labels)
    assert (values ~= nil, "Viewer: values for the color bar are empty")
    index = colorbar.index (name)
    slices = #values
    assert (brewer.rgb[index][slices] ~= nil, 
            "Viewer: number of slices not supported for this color in Color Brewer")
        
    local colorBar = {}
    for i = 1, slices do
        if (labels ~= nil) then 
            table.insert (colorBar, {value = values[i], 
                                     color = brewer.rgb[index][slices][i], 
                                     label = labels[i]})
        else
            table.insert (colorBar, {value = values[i], 
                                     color = brewer.rgb[index][slices][i]})
        end
    end
    return colorBar
end

--- function to define a color bar to be later created
--  returns a valid colorbar for TerraME
--  @param view  the set of arguments for the view
--  @param view.grouping - type of grouping ("uniquevalue", "equalsteps", "quantil", "stdeviation"
--  @param view.values   - for "uniquevalue", values must be provided
--  @param view.colors   - a valid set of color names for the Color Brewer
--  @param view.labels   - labels to be used for the colors (optional)
--  @param view.maximum  - for "equalsteps" and "quantil", maximum value must be provided
--  @param view.minimum  - for "equalsteps" and "quantil", minimum value must be provided
--  @param view.slices   - for "equalsteps" and "quantil", number of slices must be provided
--  @usage view.defColorBar (v)
--
--  view.mkColorBar (myview)
--
colorbar.make = function (view)
    -- test the grouping 
    local colorBar = {}
    if (view.grouping == "uniquevalue") then
        assert (view.values     ~= nil, "Viewer: possible values of map must be provided")
        assert (view.colors     ~= nil, "Viewer: names of colors must be provided")
        -- create the colorbar
        colorBar   = colorbar.brewer (view.values, view.colors, view.labels)
    else
        assert (view.slices  ~= nil, "Viewer: number of slices must be provided")
        assert (view.minimum ~= nil, "Viewer: minimum value must be provided")
        assert (view.maximum ~= nil, "Viewer: maximum value must be provided")
        -- TerraME will take care of getting the right limits for the values
        -- Here we just want to create a valid colorBar
        -- The values used for the colorBar will be overridden by TerraME
        local values = {}
        for i = 1, view.slices do values[i] = i end
        colorBar  = colorbar.brewer(values, view.colors, view.labels)
    end
    return colorBar
end

return colorbar